<?php defined('SYSPATH') || die('No direct script access.');

/**
 * Class Model_Plan
 *
 * @property-read   $ID             int
 * @property        $ExternalID     string
 * @property        $FacultyID      int
 * @property        $SemesterID     int
 * @property        $GradeID        int
 */
class Model_Plan extends Model_Container
{
    protected function getRawData($id) {
        $sql = 'CALL `Plan_GetInfo`(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('Study plan not found');

        return $info->offsetGet(0);
    }

    public static function make() {
        return new Model_Helper_PlanBuilder();
    }

    protected function create() {
        $sql = 'SELECT `Plan_Create`(ExternalID, Year, FacultyID) AS `ID`';
        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()->get('ID');
    }

    /**
     * @param string $extid
     * @return int
     */
    public static function withExternalID($extid) {
        $sql = 'SELECT `Plan_GetIDFromExternalID`(:extid) AS `ID`';
        $id = DB::query(Database::SELECT, $sql)
            ->param(':extid', $extid)
            ->execute()->get('ID');
        return (int)$id;
    }

    /** @return Model_Discipline[] */
    public function getDisciplines($semesterID = null) {
        $sql = 'CALL Plan_GetDisciplines(:id, :semester)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semester', $semesterID)
            ->execute();

        $disciplines = array();
        foreach ($info as $disciplineID) {
            $disciplines[] = Model_Discipline::load($disciplineID);
        }

        return $disciplines;
    }

    /** @return Model_Teacher[] */
    public function getTeachers($loadAll = false, $semesterID = null) {
        $sql = 'CALL Plan_GetTeachers(:id, :semester, :loadAll)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semester', $semesterID)
            ->param(':loadAll', $loadAll)
            ->execute();

        $teachers = array();
        foreach ($info as $teacherID) {
            $teachers[] = Model_Teacher::load($teacherID);
        }

        return $teachers;
    }

    public function update() {
        $this->create();
    }

    public function bindDiscipline($disciplineID) {
        $sql = 'SELECT `Plan_BindDiscipline`(:id, :discipline) AS `ID`';
        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':discipline', $disciplineID)
            ->execute();
    }
}
